## no critic(RequireArgUnpacking, RequireParenthesesLastChar)
## no critic(RequireExtendedFormatting)
## no critic(ProhibitMultiplePackages)

## no critic(ProhibitNoWarnings)
## no critic(RequireBlockMap, RequireBlockGrep)
## no critic(RequireInterpolationOfMetachars)
## no critic (ProhibitMagicNumbers)
## no critic (RequireCommentForExportedFunctions)
use strict;
use warnings FATAL => 'all';
no warnings qw/uninitialized redefine/;
package Order;
use base 'Test::Class';

use Test::More;
use List::Util qw/sum/;

use DE_EPAGES::Billsafe::API::Request;

use FindBin '$Bin';
use lib $Bin;
use BillsafeTest;

use constant DIAG  => 0;
use constant DIAG2 => 0;

# Test::Class, RUNNING ORDER OF METHODS

# BillSafe Errors (BSE)
# =====================
# BSE221: Parameter article_number must be unique
# BSE216: Parameter ArticleList has invalid parameters - ???
# BSE212: Parameter articleList has missing parameters - ???

sub get_articles {
    my( $self ) = @_;
    return map Billsafe::Article->new( $_ ),
    {name => 'K', number => 444, grossPrice =>  1.10, quantity => 3.5, tax => 19, type => 'goods'},
    {name => 'A', number => 111, grossPrice =>  4.29, quantity => 4,   tax => 19, type => 'goods'},
    {name => 'B', number => 222, grossPrice => 19.99, quantity => 1,   tax =>  7, type => 'goods'},
    {name => 'C', number => 333, grossPrice =>  0.79, quantity => 2,   tax => 19, type => 'goods'};
}

sub expected_total { 1.10 * 3.5 + 4 * 4.29 + 1 * 19.99 + 2 * 0.79 }
sub expected_tax   { 4.91456373203487 }

sub startup : Test(startup) {
    my( $self ) = @_;
    DIAG and Test::More::diag 'startup';
    return;
}

sub shutdown : Test(shutdown) {
    my( $self ) = @_;
    DIAG and Test::More::diag 'shutdown';
    return;
}

sub setup : Test(setup) {
    my( $self ) = @_;
    DIAG and Test::More::diag 'setup';

    my $account = Billsafe::Account->epages_merchant;
    $self->{billsafe} =
        Billsafe::Request->new( account => $account, logger => \&diag, silent => 1 );

    my $order = Billsafe::Order->new;
    $order->add_article( $_ ) for $self->get_articles;
    $self->{order} = $order;
    $self->{customer} = Billsafe::Customer->sandbox_paul;
    return;
}

sub after : Test(teardown) {
    DIAG and Test::More::diag 'teardown';
}

sub t010 : Test(no_plan) {
    my( $self ) = @_;

    isa_ok $self->{billsafe}, 'DE_EPAGES::Billsafe::API::Request'
        or $self->BAILOUT( 'wrong ISA' );

    my $order = $self->{order};
    isa_ok $order, 'Billsafe::Order' or $self->BAILOUT( 'wrong ISA' );

    my $articles = $order->{articles};
    for ( @$articles ) {
        isa_ok $_, 'Billsafe::Article' or $self->BAILOUT( 'wrong ISA' );
    }

    my $total = sum map $_->get_price, @$articles;
    is $total, $self->expected_total, 'calculation of total as expected'
        or Test::More::diag explain $articles;
    is $order->calc_amount, $total, 'calculation of total'
        or Test::More::diag explain $order;

    my $totaltax = sum map $_->get_tax_amount, @$articles;
    is $totaltax, $self->expected_tax, 'calculation of tax as expected'
        or Test::More::diag explain $articles;
    is $totaltax, $self->expected_tax, 'calculation of tax'
       or Test::More::diag explain $articles;
    $order->tax($self->expected_tax);


    DIAG and Test::More::diag explain $order;

    my %nvp = $order->as_nvp;
    DIAG2 and Test::More::diag explain \%nvp;

    isa_ok $self->{customer}, 'Billsafe::Customer'
        or $self->BAILOUT( 'wrong ISA' );

    return;
}

package Order2; # andere Artikelliste
use base 'Order';
# perl -c:
# cannot test anonymous subs -
# you probably loaded a Test::Class too late (after the CHECK block was run).
# See 'A NOTE ON LOADING TEST CLASSES' in perldoc Test::Class for more details

sub get_articles {
    my( $self ) = @_;
    return map Billsafe::Article->new( $_ ),
    {name => 'K', number => 444, grossPrice => 0.90, quantity => 0.5, tax => 19, type => 'goods'},
    {name => 'A', number => 111, grossPrice => 4.99, quantity => 1,   tax =>  7, type => 'goods'},
    {name => 'C', number => 333, grossPrice => 0.79, quantity => 2,   tax => 19, type => 'goods'};
}

sub expected_total { 0.45 + 1 * 4.99 + 2 * 0.79 }
sub expected_tax   { 0.650566245189665 }

package Order3; # Nettopreis
use base 'Order';

sub get_articles {
    my( $self ) = @_;
    return map Billsafe::Article->new( $_ ),
    {name => 'K', number => 444, netPrice => 0.80, quantity => 7.7, tax => 19, type => 'goods'},
    {name => 'A', number => 111, netPrice => 4.80, quantity => 1,   tax =>  7, type => 'goods'},
    {name => 'C', number => 333, netPrice => 0.90, quantity => 6,   tax => 19, type => 'goods'};
}

sub expected_total { 6.16 + 1.1704 + 4.8 + 0.336 + 5.4 + 1.026 }
sub expected_tax   {        1.1704 +     + 0.336 +       1.026 }

1;
