package DE_EPAGES::Catalog::t::CategorySequenceResource; ## no critic (RequireCommentForExportedFunctions)
use base DE_EPAGES::Shop::API::Test;

use strict;
use Data::Dumper;
use DE_EPAGES::Catalog::API::Resource::CategorySequence;
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use Test::Exception;
use Test::More;

## no critic qw(ProhibitMagicNumbers)

sub init {
    my $self = shift;
    $self->SUPER::init;

    # set up test data
    my $Shop = $self->shop;
    $Shop->set( {
            DomainName        => 'test-domain',
            HasSSLCertificate => 1
        }
    );
    $Shop->featureMaxValue( 'Pages',         5 );
    $Shop->featureMaxValue( 'Subcategories', 2 );
    my $Categories = $Shop->child('Categories');
    my $TestCat    = $Categories->insertChild(
        'Category',
        {
            'Alias'     => 'aliasRoot',
            'IsVisible' => 1,
        }
    );

    $TestCat->set( {
            'Description' => 'SomeText',
            'Name'        => 'SomeName',
        },
        GetPKeyLanguageByCode('en')
    );

    # Add categories as children to TestCat.
    $TestCat->insertChild(
        'Category',
        {
            'Alias'     => 'alias1',
            'IsVisible' => 1,
            'Position'  => 10,
        }
    );
    $TestCat->insertChild(
        'Category',
        {
            'Alias'     => 'alias2',
            'IsVisible' => 1,
            'Position'  => 20,
        }
    );
    $TestCat->insertChild(
        'Category',
        {
            'Alias'     => 'alias3',
            'IsVisible' => 0,
            'Position'  => 30,
        }
    );
}

sub test {
    my $self = shift;
    my $Shop = $self->shop;

    my $Resource  = DE_EPAGES::Catalog::API::Resource::CategorySequence->new;
    my $Category  = $Shop->child('Categories')->child('aliasRoot');

    ## test GET
    my $hResult = $Resource->get($Category);
    is_deeply $hResult,
      ['alias1', 'alias2', 'alias3'],
      'GET categorySequence'
      or Test::More::diag Dumper($hResult);

    ## test PUT
    $hResult = $Resource->put($Category, ['alias2', 'alias1']);
    is_deeply $hResult,
      ['alias2', 'alias1', 'alias3'],
      'PUT categorySequence'
      or Test::More::diag Dumper($hResult);

    ## test PUT invisible
    $hResult = $Resource->put($Category, ['alias2', 'alias3', 'alias1']);
    is_deeply $hResult,
      ['alias2', 'alias3', 'alias1'],
      'PUT categorySequence including invisible categories'
      or Test::More::diag Dumper($hResult);

    ## test PUT 2nd attempt
    $hResult = $Resource->put($Category, ['alias1', 'alias2']);
    is_deeply $hResult,
      ['alias1', 'alias3','alias2'],
      'PUT categorySequence with another combination'
      or Test::More::diag Dumper($hResult);

    ## test BadRequest
    throws_ok(sub { $Resource->put($Category, ['different_alias', 'alias2']) }, qr/BadRequest/, 'BadRequest with unknown category name!');
    throws_ok(sub { $Resource->put($Category, ['alias2','alias2']) }, qr/BadRequest/, 'BadRequest with a duplicate category');
}

__PACKAGE__->runScript;
done_testing();
