#========================================================================================
# package      DE_EPAGES::ContextualAppAds::Hooks::ContentObject
# state        public
# description  Hook functions of the cartridge ContextualAppAds.
#========================================================================================
package DE_EPAGES::ContextualAppAds::Hooks::ContentObject;

use strict;
use DE_EPAGES::ContextualAppAds::UI::JSON qw ( JSONWriteAppData );

#========================================================================================
# function     OnUnityView
#----------------------------------------------------------------------------------------
# syntax       OnUnityView( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_ContentObject_UnityView.
#               Writes the JSON data of the context sensitive apps.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameter
#               <ul>
#                   <li>Servlet - servlet object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnUnityView {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    JSONWriteAppData($Servlet, 'product');

    return;
}

#========================================================================================
# function     OnUnityViewPagedChildren
#----------------------------------------------------------------------------------------
# syntax       OnUnityViewPagedChildren( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_ContentObject_UnityViewPagedChildren.
#               Writes the JSON data of the context sensitive apps.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameter
#               <ul>
#                   <li>Servlet - servlet object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnUnityViewPagedChildren {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    JSONWriteAppData($Servlet, 'product') unless $Servlet->params('DELETE');

    return;
}

1;
