# Usage example:
#
#   nmake -f ../Core/Scripts/Core.mak \
#		COMPANY=DE_EPAGES CARTRIDGE=Core PERL=$(PERL) test
#

COMPANY = DE_EPAGES
CARTRIDGE = Core
PERL = perl
DE_EPAGES = ../../DE_EPAGES

RUNTEST	  = $(PERL) $(DE_EPAGES)/Installer/Scripts/runTests.pl
RUNBROWSER = $(PERL) $(DE_EPAGES)/Core/Scripts/browser.pl
CARTRIDGE_COVER_DB = $(EPAGES_STATIC)/coverage/$(COMPANY)-$(CARTRIDGE)/cover_db

COVER=$(dir $(PERL))/cover
PROVE=$(dir $(PERL))/prove
PROVE_ARGS=--timer --jobs 9 --rules='par=./t/unit/**'
CRITIC=$(PERL) $(DE_EPAGES)/Core/Scripts/critic.pl
CRITIC_CONF="$(DE_EPAGES)/Core/Scripts/perlcritic_hard.conf"
test:
	$(RUNTEST) -cartridge $(COMPANY)::$(CARTRIDGE) -storename $(STORE)

regression_test:
	$(RUNTEST) -cartridge DE_EPAGES::Test -storename $(STORE)

unit_test:
	$(PROVE) --jobs 9 t/unit $(PROVE_ARGS)

integration_test:
	EPAGES_TESTSITE=$(STORE) $(PROVE) t/integration $(PROVE_ARGS)

test_coverage: makefile
	$(MAKE) --ignore-errors collect_coverage STORE=$(STORE) COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE)
	$(MAKE) --ignore-errors coverage_report COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE)
	$(RUNBROWSER) $(CARTRIDGE_COVER_DB)/coverage.html

collect_coverage: makefile
	$(MKDIR) $(CARTRIDGE_COVER_DB)
	$(COVER) $(CARTRIDGE_COVER_DB) -delete
	$(MAKE)  --ignore-errors test COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE) STORE=$(STORE) HARNESS_PERL_SWITCHES=-MDevel::Cover DEVEL_COVER_OPTIONS=+select,$(COMPANY)/$(CARTRIDGE)/,+ignore,.*,-coverage,statement,subroutine,-subs_only,1,-db,$(CARTRIDGE_COVER_DB)

coverage_report:
	$(COVER) $(CARTRIDGE_COVER_DB) $(COVER_ARGS)

# critic.pl uses Test::More to print output. We need to normalize this for sonar.
critic_report:
	$(RM) -f sonar_perlcritic_report.txt
	$(CRITIC) --profile $(CRITIC_CONF) --format sonar . 2>&1 | egrep -e ^# | sed -e 's/^..//' > sonar_perlcritic_report.txt || true

sonar_properties:
	$(PERL) $(DE_EPAGES)/Core/Scripts/generateSonarProperties.pl -cartridge=$(CARTRIDGE) -company=$(COMPANY)

sonar_reports: makefile
	$(RM) -f sonar_test_report.tgz
	EPAGES_TESTSITE=$(STORE) $(PROVE) -r -a sonar_test_report.tgz $(PROVE_ARGS) || true
	$(MAKE) --ignore-errors collect_coverage STORE=$(STORE) COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE)
	$(MAKE) --ignore-errors coverage_report COVER_ARGS="-report clover" COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE) CARTRIDGE_COVER_DB=$(CARTRIDGE_COVER_DB)
	$(RM) -f sonar_coverage.xml
	$(CP) $(CARTRIDGE_COVER_DB)/clover.xml sonar_coverage.xml
	$(MAKE) --ignore-errors critic_report

sonar_scan: makefile
	$(MAKE) --ignore-errors sonar_properties COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE)
	sonar-scanner $(SONAR_SCANNER_ARGS)
