use strict;
use Test::Simple tests => 133;
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

sub Main {

    ok( 1, 'init');

    my $Now = GetCurrentDateTime();
    foreach my $LocaleID (qw (es_ES de_DE de_CH de_AT en_AU en_GB en_US fr_FR it_IT nl_NL sv_SE)) {
        my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=> $LocaleID);
        foreach my $DateType (qw (date time datetime)) {
            foreach my $Length (qw (short medium long full )) {
                eval {
                    my $FormattedValue = $Formatter->format_datetime($Now, $DateType, $Length);

                    my $NowDec = $Formatter->parse_datetime($FormattedValue, $DateType, $Length);
                    my $DecFormattedValue = $Formatter->format_datetime($NowDec, $DateType, $Length);

                    if ($DateType eq 'date' and $Length eq 'medium') {
                        # <DateTime::Format::Strptime::VERSION = '1.54'>
                        # When abbreviating, sometimes there's a period, sometimes not.
                        # $month =~ s/\.$//;
                        # </>
                        $FormattedValue =~ s,^([0-9]+\s+\S+)\.(\s+.*),$1$2,;
                        $DecFormattedValue =~ s,^([0-9]+\s+\S+)\.(\s+.*),$1$2,;
                    }


                    my $Test = $FormattedValue eq $DecFormattedValue ? 1 : 0;
                    ok ($Test, sprintf ('%s:%s:%s=%s/%s', $LocaleID, $DateType, $Length,
                                                          $FormattedValue, $DecFormattedValue));
                };
                if (ExistsError()) {
                    ok (0, sprintf ('%s:%s:%s = %s', $LocaleID, $DateType, $Length,
                                                     $Formatter->formatString($DateType, $Length)));
                }
            }
        }
    }
}

RunScript(Sub=>\&Main, Debug=>1);
