﻿use Test::More tests => 20;

use strict;
use utf8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::IDN qw(
    EncodeDomain
    DecodeDomain
    EncodeDomainInUrl
    DecodeDomainInUrl
    EncodeDomainInEMailAddress
    DecodeDomainInEMailAddress
    AreDomainsEqual
    IsValidIDN
);

sub Main {
    RunTests( \&EncodeDomain, 'EncodeDomain', [
        [ 'müller.de' ], [ 'xn--mller-kva.de' ],
        [ 'MÜLLER.de' ], [ 'xn--mller-kva.de' ],
        [ 'MuELlEr.de' ], [ 'mueller.de' ],
        [ 'äöüß.de' ], [ 'xn--ss-uia6e4a.de' ],
        [ 'Sørensen.dk' ], [ 'xn--srensen-q1a.dk' ],
        [ 'BÜcher.müLLEr.de' ], [ 'xn--bcher-kva.xn--mller-kva.de' ],
        [ 'waßER.de' ], [ 'wasser.de' ],
              ]);
    RunTests( \&DecodeDomain, 'DecodeDomain', [
        [ 'xn--mller-kva.de' ], [ 'müller.de' ],
              ]);
    RunTests( \&EncodeDomainInUrl, 'EncodeDomainInUrl', [
        [ 'https://shop.müller.de/shop1.admin' ], [ 'https://shop.xn--mller-kva.de/shop1.admin' ],
        [ 'https://BÜcher:müLLEr@shop.müller.de/shop1.admin' ], [ 'https://B%C3%9Ccher:m%C3%BCLLEr@shop.xn--mller-kva.de/shop1.admin' ],
              ]);
    RunTests( \&DecodeDomainInUrl, 'DecodeDomainInUrl', [
        [ 'https://shop.xn--mller-kva.de/shop1.admin' ], [ 'https://shop.müller.de/shop1.admin' ],
        [ 'https://BÜcher:müLLEr@shop.xn--mller-kva.de/shop1.admin' ], [ 'https://B%C3%9Ccher:m%C3%BCLLEr@shop.müller.de/shop1.admin' ],
              ]);
    RunTests( \&EncodeDomainInEMailAddress, 'EncodeDomainInEMailAddress', [
        [ 'webmaster@müller.de' ], [ 'webmaster@xn--mller-kva.de' ],
        [ 'web(@).mueller@müller.de' ], [ 'web(@).mueller@xn--mller-kva.de' ],
              ]);
    RunTests( \&DecodeDomainInEMailAddress, 'DecodeDomainInEMailAddress', [
        [ 'webmaster@xn--mller-kva.de' ], [ 'webmaster@müller.de' ],
        [ 'web(@).mueller@xn--mller-kva.de' ], [ 'web(@).mueller@müller.de' ],
              ]);
    RunTests( \&AreDomainsEqual, 'AreDomainsEqual', [
        [ 'müller.de', 'xn--mller-kva.de' ], [ 1 ],
              ]);
    RunTests( \&IsValidIDN, 'IsValidIDN', [
        [ 'www.vanitydomainshop.de' ], [ 1 ],
        [ 'MÜLLER.de' ], [ 1 ],
        [ 'BÜcher.müLLEr.de' ], [ 1 ],
              ]);
}

RunScript( Sub => \&Main, Message => 'long' );

sub RunTests {
    my( $cRun, $TestName, $Tests ) = @_;

    for (my $i = 0; $i < scalar(@$Tests); $i += 2) {
        my $Parameters = $Tests->[$i];
        my $ExpectedResult = $Tests->[$i+1];
        my @ActualResult = $cRun->(@$Parameters);

        my $OK = @$ExpectedResult ~~ @ActualResult;
        $TestName = $TestName.'('.join(', ',@$Parameters).')';

        ok($OK, $TestName) or Test::More::diag(explain({
                'Expected' => $ExpectedResult,
                'Actual' => \@ActualResult}
        ));
    }
}
