#========================================================================================
# §package      DE_EPAGES::CrossSelling::t::CrossSellingResource
# §state        public
# §base         DE_EPAGES::Shop::API::TestImportXML
# §description  tests the REST resource for product-crossselling.
#========================================================================================
package DE_EPAGES::CrossSelling::t::ManualCrossSellingResource;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::CrossSelling::API::Resource::CrossSelling;
use Data::Dumper qw(Dumper);

use Test::More;

use strict;

sub shopAlias {
    return 'TestShop';
}
sub importXMLPath {
    return 'importManualCrossSelling.xml';
}

sub test {
    my $self = shift;
    $self->testGetCrossSelling();
    $self->testGetCrossSellingPaging();
    done_testing();
}

sub testGetCrossSelling {
    my $self = shift;
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });
    my $LocaleID = 'en_GB';

    my $Resource = DE_EPAGES::CrossSelling::API::Resource::CrossSelling->new();

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('Product1');
    my $ProductGUID = $Product->guid;

    my $hCrossSellingProducts = {};

    foreach my $i (1..5) {
      $hCrossSellingProducts->{$i} = $ProductFolder->child('CrossSellingProduct'.$i)->guid;
    }

    my $hData = $Resource->get( $Product, $LocaleID );

    # we have a lot of elements and hash elements in items
    # we can not compare all thing. So we shorten the hash and compare only the productId
    # of the elements
    my $aCrossSellingProductIDs = [];
    foreach my $hEmbeddedProduct (@{$hData->{'embedded'}->{'items'}}){
      push @$aCrossSellingProductIDs, { 'productId' => $hEmbeddedProduct->{'productId'}};
    }

    $hData->{'embedded'}->{'items'} = $aCrossSellingProductIDs;

    is_deeply ($hData,{
        'embedded' => {
          'page' => 1,
          'items' => [
            {
              'productId' => $hCrossSellingProducts->{1}
            },
            {
              'productId' => $hCrossSellingProducts->{2}
            },
            {
              'productId' => $hCrossSellingProducts->{3}
            },
            {
              'productId' => $hCrossSellingProducts->{4}
            },
            {
              'productId' => $hCrossSellingProducts->{5}
            }

          ],

          'resultsPerPage' => 100,
          'results' => 5
        },
        'links' => [
          {
            'rel' => 'self',
            'href' => "https://test-domain/api/products/$ProductGUID/crossselling/",
            'title' => undef
          },
          {
            'rel' => 'product',
            'href' => "https://test-domain/api/products/$ProductGUID",
            'title' => undef
          }
        ],
        'manualCrossSellingTitle' => undef
      },
      'GET crossselling')
      or Test::More::diag Dumper($hData);

    return;
}

# test paging by getting the last 2 items
sub testGetCrossSellingPaging {
    my $self = shift;
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });
    my $LocaleID = 'en_GB';

    my $Resource = DE_EPAGES::CrossSelling::API::Resource::CrossSelling->new();

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('Product1');
    my $ProductGUID = $Product->guid;

    my $hCrossSellingProducts = {};

    foreach my $i (1..5) {
      $hCrossSellingProducts->{$i} = $ProductFolder->child('CrossSellingProduct'.$i)->guid;
    }

    my $hData = $Resource->get( $Product, $LocaleID, undef, {'PageSize' => 3, 'Page' => 2} );

    # we have a lot of elements and hash elements in items
    # we can not compare all thing. So we shorten the hash and compare only the productId
    # of the elements
    my $aCrossSellingProductIDs = [];
    foreach my $hEmbeddedProduct (@{$hData->{'embedded'}->{'items'}}){
      push @$aCrossSellingProductIDs, { 'productId' => $hEmbeddedProduct->{'productId'}};
    }

    $hData->{'embedded'}->{'items'} = $aCrossSellingProductIDs;

    is_deeply ($hData,{
        'embedded' => {
          'page' => 2,
          'items' => [
            {
              'productId' => $hCrossSellingProducts->{4}
            },
            {
              'productId' => $hCrossSellingProducts->{5}
            }

          ],
          'resultsPerPage' => 3,
          'results' => 5
        },
        'links' => [
          {
            'rel' => 'self',
            'href' => "https://test-domain/api/products/$ProductGUID/crossselling/",
            'title' => undef
          },
          {
            'rel' => 'product',
            'href' => "https://test-domain/api/products/$ProductGUID",
            'title' => undef
          }
        ],
        'manualCrossSellingTitle' => undef
      },
      'GET crossselling with paging')
      or Test::More::diag Dumper($hData);

    return;
}



__PACKAGE__->runScript();

1;