#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::PackstationRequest
#----------------------------------------------------------------------------------------
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Packstation request.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::PackstationRequest;

use base DE_EPAGES::Core::API::Test;
use Data::Dumper;
use DE_EPAGES::DHLIntraship::API::Constants qw ( $GERMAN_LOCALE );
use DE_EPAGES::DHLIntraship::API::SOAP::PackstationRequest;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::MockObject::Extends;
use Test::More;

use strict;

sub test {
    my $self = shift;
    $self->packstationRequest_positive_test();
    $self->packstationRequest_operationName_noExist();
    done_testing();
}

sub packstationRequest_positive_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServlet                   = $self->_getMockedServlet();
    my $MockedOperationName             = $self->_getMockedOperationName();
    my $MockedPackstationRequest        = $self->_createPackstationRequest($MockedServlet);
    my $hObtainedRequest                = $MockedPackstationRequest->assemble($MockedOperationName,$MockedServlet);
    my $hExpectedRequest                = $self->_getExpectedRequest();

    is_deeply( $hObtainedRequest, $hExpectedRequest, "$SubTestName - Packstation request message." )
      or Test::More::diag Dumper($hObtainedRequest);

    return;
}

sub packstationRequest_operationName_noExist {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServlet                   = $self->_getMockedServlet();
    my $MockedOperationName             = $self->_getMockedOperationNameNotExist();
    my $MockedPackstationRequest        = $self->_createPackstationRequest($MockedServlet);
    my $hObtainedRequest                = $MockedPackstationRequest->assemble($MockedOperationName,$MockedServlet);
    my $ExpectedRequest = undef;
    is($hObtainedRequest,$ExpectedRequest, "$SubTestName - Operation name is undefined");

    return;
}

#----------------------------------------------------------------------------------------
sub _getMockedOperationName {
    my $self = shift;
    return 'getPackstationsByAddress';
}

#----------------------------------------------------------------------------------------
sub _getMockedOperationNameNotExist {
    my $self = shift;
    return 'getPackstationsByAddressFAIL';
}

#----------------------------------------------------------------------------------------
sub _createPackstationRequest {
    my $self = shift;
    my ($Servlet) = @_;

    my $aParameterList = [$Servlet];
    my $MockedPackstationRequest = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::DHLIntraship::API::SOAP::PackstationRequest',
        $aParameterList );

    my $packstation = $self->_getExpectedRequest();
    $MockedPackstationRequest->addMockWithReturnValue( '_setPackstationsByAddress',$packstation );

    return $MockedPackstationRequest->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _getExpectedRequest {
    my $self = shift;

    my $hObtainedRequest = {
        'key'  => '',
        'address' => {
            'street'      => 'Street',
            'streetNo'    => 'StreetNo',
            'zip'         => 'ZipCode',
            'city'        => 'City',
            'countryCode' => $GERMAN_LOCALE
        }
    };

    return $hObtainedRequest;
}

#----------------------------------------------------------------------------------------
sub _getMockedServlet {
    my $self = shift;

    my $MockifyShippingFulfillmentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship', {}
    );
    my $MockedShippingFulfillmentMethod = $MockifyShippingFulfillmentMethod->getMockObject();

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedShippingFulfillmentMethod);

    my $hAddressParams = {
        'Street'      => 'Street',
        'StreetNo'    => 'StreetNo',
        'ZipCode'     => 'ZipCode',
        'City'        => 'City'
    };
    $MockifyServlet->addParams($hAddressParams);

    return $MockifyServlet->getMockObject();
}


__PACKAGE__->runScript();
