#!perl -w
use Test::More tests => 8;
use strict;
use Text::Diff qw( diff );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Dictionary::API::Localizer qw ( Replace );

my $hStrings = {
    text1 => 'Text_1',
    text2 => 'Text_2',
    var => 'DONT_REPLACE_var',
    each => 'DONT_REPLACE_each',
    longText => "<b>I'll be back</b>\nlater",
    x => 'single_char',
    '123456' => 'DONT_REPLACE_all_digits',
    '1und1' => 'leading_and_trailing_digits',
    _xyz => 'leading_underscore',
};

my $MasterTemplate = <<'END_TEMPLATE';
Start
  {text1} {{each}}
  ${var} {text2}
  {x} {123456} {_xyz} {1und1}
  {unknownTag}
  <script language="text/javascript">
    var text = '{longText}';
  </script>
  {longText}
  {longText[html]}
  <script type="text/x-jquery-tmpl">
    {longText}
  </script>
  <script type = 'jquery'>
    {longText}
  </script>
End
END_TEMPLATE

my $ExpectedResult = <<'END_RESULT';
Start
  Text_1 {{each}}
  ${var} Text_2
  single_char {123456} leading_underscore leading_and_trailing_digits
  {unknownTag}
  <script language="text/javascript">
    var text = '\u003Cb\u003EI\u0027ll\u0020be\u0020back\u003C\u002Fb\u003E\u000Alater';
  </script>
  <b>I'll be back</b>
later
  &lt;b&gt;I&#39;ll be back&lt;/b&gt;
later
  <script type="text/x-jquery-tmpl">
    <b>I'll be back</b>
later
  </script>
  <script type = 'jquery'>
    <b>I'll be back</b>
later
  </script>
End
END_RESULT

RunScript( 'Sub' => sub {
    is( Replace( '', $hStrings ),  '', "empty string" );
    is( Replace( undef, $hStrings ),  '', "undef" );
    is( Replace( " \t\n", $hStrings ),  " \t\n", "whitespace" );

    my $aWarnings = BlockWarnings( sub {
        my $Result = Replace( $MasterTemplate, $hStrings );
        is( $Result, $ExpectedResult, "Replace" ) or Test::More::diag diff(\$ExpectedResult, \$Result);
    });
    is( scalar @$aWarnings, 1, "1 warning" ) or Test::More::diag join "\n", map { $_->message } @$aWarnings;
    my $Warning = $aWarnings->[0];
    is( $Warning->code, 'NOT_REPLACED', "warning code: NOT_REPLACED" );
    is( $Warning->vars->{'Name'}, 'unknownTag', "warning var: Name=unknownTag" );
    ok( $Warning->message =~ /^tag unknownTag\[\] was not replaced/, "warning message" );

});
