#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::CaptureRequest
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Capture request.
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::CaptureRequest;

use base DE_EPAGES::Core::API::Test;
use Data::Dumper;
use DE_EPAGES::DocdataPay::API::SOAP::Request::CaptureRequest;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::MockObject::Extends;
use Test::More;

use strict;

sub test {
    my $self = shift;
    $self->capture_positive_test();
    done_testing();
}

sub capture_positive_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedCaptureRequest            = $self->_createCaptureRequest();
    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay();
    my $hObtainedRequest                = $MockedCaptureRequest->buildCaptureRequest($MockedLineItemPaymentDocdataPay);
    my $hExpectedRequest                = $self->_getExpectedRequest();

    is_deeply( $hObtainedRequest, $hExpectedRequest, "$SubTestName - Capture request message." )
      or Test::More::diag Dumper($hObtainedRequest);

    return;
}

sub _createCaptureRequest {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::DocdataPay::API::SOAP::Request::CaptureRequest',
        $aParameterList );

    $Mockify->addMock( '_TestObject', sub {
        shift;
        return;
    });

    my $aParameterTypes = [ 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_convertNumberToDocdataFormat', '22295', $aParameterTypes );

    return $Mockify->getMockObject();
}

sub _createLineItemPaymentDocdataPay {
    my $self = shift;

    my $hAtributes = {
        'APIVersion'    => '1.2',
        'PaymentID'     => '4907435245',
        'PaymentMethod' => $self->_createPaymentMethodDocdataPay(),
        'Parent'        => $self->_createLineItemContainer()
    };
    my $LineItemPaymentDocdataPay =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay',
        $hAtributes );

    return $LineItemPaymentDocdataPay;
}

sub _createPaymentMethodDocdataPay {
    my $self = shift;

    my $hAtributes = {
        'MerchantName'     => 'merchantname',
        'MerchantPassword' => 'merchantpassword'
    };
    my $PaymentMethodDocdataPay = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay', $hAtributes );

    return $PaymentMethodDocdataPay;
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAtributes = {
        'CurrencyID' => 'GBP',
        'GrandTotal' => '222.95'
    };
    my $LineItemContainer =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAtributes );

    return $LineItemContainer;
}

sub _getExpectedRequest {
    my $self = shift;

    my $hObtainedRequest = {
        'version'  => '1.2',
        'merchant' => {
            'name'     => 'merchantname',
            'password' => 'merchantpassword'
        },
        'paymentId' => '4907435245',
        'amount'    => {
            'currency' => 'GBP',
            '_'        => '22295'
        }
    };

    return $hObtainedRequest;
}

__PACKAGE__->runScript();
