#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::CreateRequest
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Create request.
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::CreateRequest;

use base DE_EPAGES::Core::API::Test;
use strict;

use Data::Dumper;
use DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount );
use Readonly;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

sub test {
    my $self = shift;
    $self->_test_create_request_without_shipping_address_GenderFemale_positive();
    $self->_test_create_request_with_shipping_address_GenderMale_positive();
    $self->_test_create_request_with_shipping_address_with_GenderUndefined_positive();
    $self->_test_create_request_without_shipping_positive();
    $self->_test_missing_merchant_name_negative();
    $self->_test_missing_merchant_password_negative();
    done_testing();
}

sub _test_create_request_without_shipping_address_GenderFemale_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 1,
        'HasShippingAddress' => 0,
        'Gender' => '1',
        'PaymentMethod' => {
            'MerchantName' => 'merchantname',
            'MerchantPassword' => 'merchantpassword',
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    $hParams->{'Gender'} = 'F';
    my $ExpectedMessage     = $self->_build_create_request_message_with_shipping($hParams);
    is_deeply( $GeneratedMessage, $ExpectedMessage,
        "$SubTestName - Created hash matches expected hash." )
      or Test::More::diag Dumper($GeneratedMessage);
    is(GetCallCount($MockedCreateRequest, '_buildTotalVatAmount'), 4, "$SubTestName - _buildTotalVatAmount was called 4 times.");
    is(GetCallCount($MockedCreateRequest, '_buildLineItem'), 4, "$SubTestName - _buildLineItem was called 4 times.");

    return;
}

sub _test_create_request_with_shipping_address_GenderMale_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 1,
        'HasShippingAddress' => 1,
        'Gender' => '0',
        'PaymentMethod' => {
            'MerchantName' => 'merchantname',
            'MerchantPassword' => 'merchantpassword',
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    $hParams->{'Gender'} = 'M';
    my $ExpectedMessage     = $self->_build_create_request_message_with_shipping($hParams);
    is_deeply( $GeneratedMessage, $ExpectedMessage, "$SubTestName - Created hash matches expected hash." )
      or Test::More::diag Dumper($GeneratedMessage);
    is(GetCallCount($MockedCreateRequest, '_buildTotalVatAmount'), 4, "$SubTestName - _buildTotalVatAmount was called 4 times.");
    is(GetCallCount($MockedCreateRequest, '_buildLineItem'), 4, "$SubTestName - _buildLineItem was called 4 times.");

    return;
}

sub _test_create_request_with_shipping_address_with_GenderUndefined_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 1,
        'HasShippingAddress' => 1,
        'Gender' => undef,
        'PaymentMethod' => {
            'MerchantName' => 'merchantname',
            'MerchantPassword' => 'merchantpassword',
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    $hParams->{'Gender'} = 'U';
    my $ExpectedMessage     = $self->_build_create_request_message_with_shipping($hParams);
    is_deeply( $GeneratedMessage, $ExpectedMessage, "$SubTestName - Created hash matches expected hash." )
      or Test::More::diag Dumper($GeneratedMessage);
    is(GetCallCount($MockedCreateRequest, '_buildTotalVatAmount'), 4, "$SubTestName - _buildTotalVatAmount was called 4 times.");
    is(GetCallCount($MockedCreateRequest, '_buildLineItem'), 4, "$SubTestName - _buildLineItem was called 4 times.");

    return;
}

sub _test_create_request_without_shipping_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 0,
        'HasShippingAddress' => 0,
        'Gender' => undef,
        'PaymentMethod' => {
            'MerchantName' => 'merchantname',
            'MerchantPassword' => 'merchantpassword',
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    $hParams->{'Gender'} = 'U';
    my $ExpectedMessage     = $self->_build_create_request_message_without_shipping($hParams);
    is_deeply( $GeneratedMessage, $ExpectedMessage, "$SubTestName - Created hash matches expected hash." )
      or Test::More::diag Dumper($GeneratedMessage);
    is(GetCallCount($MockedCreateRequest, '_buildTotalVatAmount'), 3, "$SubTestName - _buildTotalVatAmount was called 3 times.");
    is(GetCallCount($MockedCreateRequest, '_buildLineItem'), 3, "$SubTestName - _buildLineItem was called 3 times.");

    return;
}

sub _test_missing_merchant_name_negative {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 0,
        'HasShippingAddress' => 0,
        'Gender' => undef,
        'PaymentMethod' => {
            'MerchantName' => undef,
            'MerchantPassword' => 'merchantpassword',
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    throws_ok { $MockedCreateRequest->buildCreateRequest($Basket) } qr/MerchantNameNotSet/,
      "$SubTestName - Expected MerchantNameNotSet exception.";

    return;
}

sub _test_missing_merchant_password_negative {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hParams = {
        'HasShipping' => 0,
        'HasShippingAddress' => 0,
        'Gender' => undef,
        'PaymentMethod' => {
            'MerchantName' => 'merchantname',
            'MerchantPassword' => undef,
        },
    };
    my $Basket              = $self->_getMockedBasket($hParams);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    throws_ok { $MockedCreateRequest->buildCreateRequest($Basket) } qr/MerchantPasswordNotSet/,
      "$SubTestName - Expected MerchantPasswordNotSet exception.";

    return;
}

#----------------------------------------------------------------------------------------

sub _getMockedCreateRequest {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest');

    $Mockify->addMockWithReturnValue( '_GetCodeByLanguageID', 'en' );

    $Mockify->addMock( '_TestObject', sub {
        shift;
        return;
    });

    $Mockify->addMock('_convertNumberToDocdataFormat', sub {
        shift;
        my ($Number) = @_;
        my $IndexDotSeparator = index( $Number, '.' );
        my $Decimal = ( $IndexDotSeparator >= 0 ) ? ( substr( $Number, $IndexDotSeparator + 1 ) . '0' ) : '00';
        $Decimal = substr( $Decimal, 0, 2 );
        return int($Number) . $Decimal;
    });

    my $aParameterTypes = ['string'];
    $Mockify->addMockWithReturnValueAndParameterCheck('_removeDashesFromGUID', '5502D366B26703F3060AD580920FF5BF',$aParameterTypes);

    $aParameterTypes = [{'object' => 'DE_EPAGES::Order::API::Object::Basket'}];
    $Mockify->addMethodSpyWithParameterCheck('buildCreateRequest', $aParameterTypes);

    $aParameterTypes = ['object', 'string', 'string', 'string'];
    $Mockify->addMethodSpyWithParameterCheck('_buildLineItem', $aParameterTypes);

    $aParameterTypes = ['object'];
    $Mockify->addMethodSpyWithParameterCheck('_buildTotalVatAmount', $aParameterTypes);

    $aParameterTypes = ['string'];
    $Mockify->addMethodSpyWithParameterCheck('_convertRateToInteger', $aParameterTypes);

    $aParameterTypes = ['object'];
    $Mockify->addMethodSpyWithParameterCheck('_getCode', $aParameterTypes);

    $Mockify->addMethodSpyWithParameterCheck('_getDescription', $aParameterTypes);

    $aParameterTypes = ['string', 'string'];
    $Mockify->addMethodSpyWithParameterCheck('_getDocdataNet', $aParameterTypes);

    $aParameterTypes = ['string', 'string'];
    $Mockify->addMethodSpyWithParameterCheck('_getDocdataVat', $aParameterTypes);

    $aParameterTypes = [{'object' => 'DE_EPAGES::Address::API::Object::Address'}];
    $Mockify->addMethodSpyWithParameterCheck('_getHouseNumber', $aParameterTypes);

    $aParameterTypes = ['object'];
    $Mockify->addMethodSpyWithParameterCheck('_getQuantity', $aParameterTypes);

    $aParameterTypes = ['string'];
    $Mockify->addMethodSpyWithParameterCheck('_removeAllWhiteSpaces', $aParameterTypes);

    $aParameterTypes = ['string'];
    $Mockify->addMethodSpyWithParameterCheck('_removeAllWhiteSpaces', $aParameterTypes);

    return $Mockify->getMockObject();
}

sub _getMockedBasket {
    my $self = shift;

    my ($hParams) = @_;
    my $Gender = $hParams->{'Gender'};

    my $hAttributes = {
        'Alias'             => '1000',
        'LineItemContainer' => $self->_getMockedLineItemContainer($hParams->{'HasShipping'}, $hParams->{'PaymentMethod'}),
        'BillingAddress'    => $self->_getMockedBillingAddress($Gender),
        'Customer'          => $self->_getMockedCustomer(),
        'User'              => $self->_getMockedUser(),
        'ShippingAddress'   => ($hParams->{'HasShippingAddress'}) ? $self->_getMockedShippingAddress($Gender) : undef,
        'ShopAddress'       => $self->_getMockedShopAddress()
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
}

sub _getMockedShippingAddress() {
    my $self = shift;
    my ($Gender) = @_;

    my $hAttributes = {
        'Company'     => 'Xavizon',
        'FirstName'   => 'Angela',
        'LastName'    => 'Merckel',
        'EMail'       => 'test@epages.com',
        'Gender'      => $Gender,
        'StreetOnly'  => 'Pilatuspool',
        'HouseNumber' => '2',
        'Zipcode'     => '20355',
        'City'        => 'Hamburg',
        'Country'     => { 'Code2' => 'DE', }
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _getMockedLineItemContainer {
    my $self = shift;
    my ($HasShipping, $PaymentMethod) = @_;

    my $hAttributes = {
        'Payment'        => $self->_getMockedLineItemPaymentDocdataPay($PaymentMethod),
        'GrandTotal'     => '197.3',
        'CurrencyID'     => 'GBP',
        'TotalBeforeTax' => '175.67',
        'LineItems'      => $self->_getMockedLineItems(),
        'Shipping'       => ($HasShipping) ? $self->_getMockedLineItemShipping() : undef,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
}

sub _getMockedLineItems {
    my $self = shift;

    my $hAttributes = {
        'BasePrice'     => '83.30',
        'LineItemPrice' => '83.30',
        'Quantity'      => '1.27',
        'TaxRate'       => '0.19',
        'NameOrAlias'   => 'Leatherman Survival Tool',
        'Product'       => $self->_getMockedProduct( 'Leatherman Survival Tool', 'lt_0401107001' ),
        'instanceOf'    => 'LineItemProduct'
    };
    my $FirstLineItemProduct =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes );

    my $SuperProduct = $self->_getMockedProduct();

    $hAttributes = {
        'BasePrice'     => '53.50',
        'LineItemPrice' => '107.00',
        'Quantity'      => '2',
        'TaxRate'       => '0.07',
        'NameOrAlias'   => 'Maglite',
        'Product'       => $self->_getMockedProduct( 'Maglite Survival Tool', 'lt_0401107002', $SuperProduct),
        'instanceOf'    => 'LineItemProduct'
    };
    my $SecondLineItemProduct =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes );

    $hAttributes = {
        'BasePrice'     => '2.38',
        'GUID'          => '564C778F-9332-322B-CFC0-7F0001015C40',
        'LineItemPrice' => '2.38',
        'Quantity'      => '1',
        'TaxRate'       => '0.19',
        'NameOrAlias'   => 'Gift wrap',
        'instanceOf'    => 'LineItemShippingOption'
    };
    my $LineItemShippingOption =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ShippingOption::API::Object::LineItemShippingOption', $hAttributes );

    return [ $FirstLineItemProduct, $SecondLineItemProduct, $LineItemShippingOption ];
}

sub _getMockedProduct {
    my $self = shift;
    my ( $Description, $Alias, $SuperProduct ) = @_;

    my $hAttributes = {
        'DisplayDescription' => $Description,
        'Alias'              => $Alias,
        'SuperProduct'       => $SuperProduct,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::Product', $hAttributes );
}

sub _getMockedLineItemPaymentDocdataPay {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $hAttributes = {
        'PaymentMethod' => $self->_getMockedPaymentMethodDocdataPay($PaymentMethod),
        'APIVersion'    => '1.2',
        'GUID'          => '5502D366-B267-03F3-060A-D580920FF5BF'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay',
        $hAttributes );
}

sub _getMockedPaymentMethodDocdataPay {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $hAttributes = {
        'MerchantName'     => $PaymentMethod->{'MerchantName'},
        'MerchantPassword' => $PaymentMethod->{'MerchantPassword'},
        'ProfileName'      => 'profilename'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay',
        $hAttributes );
}

sub _getMockedBillingAddress {
    my $self = shift;
    my ($Gender) = @_;

    my $hAttributes = {
        'FirstName'   => 'Angela',
        'LastName'    => 'Merckel',
        'EMail'       => 'test@epages.com',
        'Gender'      => $Gender,
        'StreetOnly'  => 'Pilatuspool',
        'HouseNumber' => '2',
        'Zipcode'     => '20355',
        'City'        => 'Hamburg',
        'Country'     => { 'Code2' => 'DE', }
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _getMockedCustomer {
    my $self = shift;

    my $hAttributes = { 'Alias' => '1034' };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Customer::API::Object::Customer', $hAttributes );
}

sub _getMockedUser {
    my $self = shift;

    my $hAttributes = {
        'LanguageID' => '2',
        'ID'         => '1034',
        'instanceOf' => 'User'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAttributes );
}

sub _getMockedLineItemShipping {
    my $self = shift;

    my $hAttributes = {
        'LineItemPrice' => '7',
        'TaxRate'       => '0.19',
        'NameOrAlias'   => 'Standard delivery',
        'GUID'          => '54D381FE-DD6D-BE14-D392-7F0001010F2A',
        'instanceOf'    => 'LineItemShipping'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemShipping',
        $hAttributes );
}

sub _getMockedShopAddress {
    my $self = shift;

    my $hAttributes = { 'VATID' => 'DE 123456' };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _build_create_request_message_with_shipping {
    my $self = shift;

    my ($hParams) = @_;

    my $ExpectedMessage = {
        'version'  => '1.2',
        'merchant' => {
            'name'     => 'merchantname',
            'password' => 'merchantpassword'
        },
        'merchantOrderReference' => '5502D366B26703F3060AD580920FF5BF',
        'paymentPreferences'     => {
            'profile'           => 'profilename',
            'numberOfDaysToPay' => '14'
        },
        'shopper' => {
            'id'   => '1034',
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'email'    => 'test@epages.com',
            'language' => { 'code' => 'en' },
            'gender'   => $hParams->{'Gender'}
        },
        'totalGrossAmount' => {
            'currency' => 'GBP',
            '_'        => '19730'
        },
        'billTo' => {
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'address' => {
                'street'      => 'Pilatuspool',
                'houseNumber' => '2',
                'postalCode'  => '20355',
                'city'        => 'Hamburg',
                'country'     => { 'code' => 'DE' }
            }
        },
        'invoice' => {
            'totalNetAmount' => {
                'currency' => 'GBP',
                '_'        => '17567'
            },
            'totalVatAmount' => [
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '111'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '1330'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '7',
                    '_'        => '700'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '038'
                },
            ],
            'item' => [
                {
                    'number'   => '1',
                    'name'     => 'Standard delivery',
                    'code'     => '54D381FE-DD6D-BE14-D392-7F0001010F2A',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Standard delivery',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '588'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '700'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '111'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '588'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '700'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '111'
                        },
                    }
                },
                {
                    'number'   => '2',
                    'name'     => 'Leatherman Survival Tool',
                    'code'     => 'lt_0401107001',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Leatherman Survival Tool',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    }
                },
                {
                    'number'   => '3',
                    'name'     => 'Maglite',
                    'code'     => 'lt_0401107002',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '2'
                    },
                    'description' => 'Default description',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '5000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '5350'
                    },
                    'vat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '350'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10700'
                    },
                    'totalVat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '700'
                        }
                    },
                },
                {
                    'number'   => '4',
                    'name'     => 'Gift wrap',
                    'code'     => '564C778F-9332-322B-CFC0-7F0001015C40',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Gift wrap',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '200'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '238'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '038'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '200'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '238'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '038'
                        }
                    },
                },
            ],
            'shipTo' => {
                'name' => {
                    'first' => 'Angela',
                    'last'  => 'Merckel'
                },
                'address' => {
                    'street'      => 'Pilatuspool',
                    'houseNumber' => '2',
                    'postalCode'  => '20355',
                    'city'        => 'Hamburg',
                    'country'     => { 'code' => 'DE' }
                }
            },
        }
    };

    if ($hParams->{'HasShippingAddress'}) {
        $ExpectedMessage->{'invoice'}->{'shipTo'}->{'address'}->{'vatNumber'} = 'DE 123456';
        $ExpectedMessage->{'invoice'}->{'shipTo'}->{'address'}->{'company'}   = 'Xavizon';
    }

    return $ExpectedMessage;
}

sub _build_create_request_message_without_shipping {
    my $self = shift;

    my ($hParams) = @_;

    my $ExpectedMessage = {
        'version'  => '1.2',
        'merchant' => {
            'name'     => 'merchantname',
            'password' => 'merchantpassword'
        },
        'merchantOrderReference' => '5502D366B26703F3060AD580920FF5BF',
        'paymentPreferences'     => {
            'profile'           => 'profilename',
            'numberOfDaysToPay' => '14'
        },
        'shopper' => {
            'id'   => '1034',
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'email'    => 'test@epages.com',
            'language' => { 'code' => 'en' },
            'gender'   => $hParams->{'Gender'}
        },
        'totalGrossAmount' => {
            'currency' => 'GBP',
            '_'        => '19730'
        },
        'billTo' => {
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'address' => {
                'street'      => 'Pilatuspool',
                'houseNumber' => '2',
                'postalCode'  => '20355',
                'city'        => 'Hamburg',
                'country'     => { 'code' => 'DE' }
            }
        },
        'invoice' => {
            'totalNetAmount' => {
                'currency' => 'GBP',
                '_'        => '17567'
            },
            'totalVatAmount' => [
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '1330'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '7',
                    '_'        => '700'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '038'
                },
            ],
            'item' => [
                {
                    'number'   => '1',
                    'name'     => 'Leatherman Survival Tool',
                    'code'     => 'lt_0401107001',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Leatherman Survival Tool',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    }
                },
                {
                    'number'   => '2',
                    'name'     => 'Maglite',
                    'code'     => 'lt_0401107002',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '2'
                    },
                    'description' => 'Default description',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '5000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '5350'
                    },
                    'vat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '350'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10700'
                    },
                    'totalVat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '700'
                        }
                    },
                },
                {
                    'number'   => '3',
                    'name'     => 'Gift wrap',
                    'code'     => '564C778F-9332-322B-CFC0-7F0001015C40',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Gift wrap',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '200'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '238'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '038'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '200'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '238'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '038'
                        }
                    },
                },
            ],
            'shipTo' => {
                'name' => {
                    'first' => 'Angela',
                    'last'  => 'Merckel'
                },
                'address' => {
                    'street'      => 'Pilatuspool',
                    'houseNumber' => '2',
                    'postalCode'  => '20355',
                    'city'        => 'Hamburg',
                    'country'     => { 'code' => 'DE' }
                }
            },
        }
    };

    return $ExpectedMessage;
}

__PACKAGE__->runScript();
