#==============================================================================
# package      DE_EPAGES::Paynet::UI::PaymentMethodPaynet
# state        private
#==============================================================================
package DE_EPAGES::Paynet::UI::PaymentMethodPaynet;
use base DE_EPAGES::Payment::UI::PaymentMethod;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Shop::API::Url qw (BuildShopUrl);

#==============================================================================
sub ViewSettings {
    my $self = shift;
    my $Servlet = shift;

    my $PaymentMethod = $Servlet->object();
    my $Shop = $PaymentMethod->getSite();

    $Servlet->vars('SuccessLink', BuildShopUrl($Shop, {
        'ChangeAction' => ['BlockMonitor', 'PaymentBacklinkPaynet'],
        'MGUID'        => '-USER_VARIABLE_5-',
        'betrag'       => '-AMOUNT-',
        'trans_id'     => '-TRANSACTION-',
        'bank_code'    => '-SENDER_BANK_CODE-',
        'country_id'   => '-SENDER_COUNTRY_ID-'
    },
    {
        'Type'          => 'sf',
        'UseSSL'        => 1,
        'UseObjectPath' => 1
    }, $Servlet));

    $Servlet->vars('CancelLink', BuildShopUrl($Shop, {
        'ViewAction' => 'ViewPaymentPaynetCancel',
        'ObjectID'   => '-USER_VARIABLE_1-'
    },
    {
        'Type'          => 'sf',
        'UseSSL'        => 1,
        'UseObjectPath' => 1
    }, $Servlet));

    $Servlet->vars('NotificationLink', BuildShopUrl($Shop, {
        'ChangeAction' => ['BlockMonitor', 'PaymentCallbackPaynet'],
        'MGUID'      => '-USER_VARIABLE_5-',
        'betrag'     => '-AMOUNT-',
        'trans_id'   => '-TRANSACTION-',
        'bank_code'  => '-SENDER_BANK_CODE-',
        'country_id' => '-SENDER_COUNTRY_ID-'
    },
    {
        'Type'          => 'sf',
        'UseSSL'        => 1,
        'UseObjectPath' => 1
    }, $Servlet));

    $Servlet->vars('BackLink', BuildShopUrl($PaymentMethod, {
        'ViewAction' => 'MBO-ViewSettings'
    },
    {
        'Type'          => 'admin',
        'UseSSL'        => 1
    }, $Servlet));

    return;
}

sub UnityViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    $self->ViewSettings($Servlet);

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    return;
}

#==============================================================================
sub SaveSettings {
    my $self = shift;
    my $Servlet = shift;
    my $PaymentMethod = $Servlet->object();

    #--- check form values (don't use type integer for form fields Paynet_CustomerID and Paynet_ProjectID because the length may exceed 9 chars)
    my $Form = $Servlet->form;
    my $hValues = $Form->form($PaymentMethod, 'PaynetSettings', 1);
    foreach my $field (qw(Paynet_CustomerID Paynet_ProjectID)) {
        if (isValid($hValues->{$field}) && $hValues->{$field} !~ /^\d+$/) {
            $Form->addFormError({
                'Reason' => 'FORMAT_NOT_INTEGER',
                'Form'   => 'PaynetSettings',
                'Name'   => $field
            });
        }
    }
    $Form->executeFormError;

    #--- standard save
    return $self->Save($Servlet);
}

1;
