package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 3;

use Data::Dumper;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::CustomAttributes;

## no critic qw(ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_get_master_de;
    $self->test_get_master_en;
    $self->test_get_sub_de;
}

sub test_get_master_de {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::CustomAttributes->new;
    my $ProductFolder = $self->shop->get('ProductFolder');

    my $Product = $ProductFolder->child('MyJacket');
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $ImageBasePath = $SSLDomain.$Product->get('WebPath');

    my $hData = $Resource->get( $Product, 'de_DE' );

    is_deeply( $hData, {
        'items' => [
            {
                'displayKey' => 'Text, sprachabhngig',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'Achtung: Restposten. Jetzt zugreifen!',
                        'value' => 'Achtung: Restposten. Jetzt zugreifen!'
                    }
                ],
                'type' => 'string',
                'key' => 'LocalizedString'
            },
            {
                'displayKey' => 'Ganze Zahl',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '123.456.789',
                        'value' => '123456789'
                    }
                ],
                'type' => 'number',
                'key' => 'Integer'
            },
            {
                'displayKey' => 'Dezimalzahl',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12.345,6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Float'
            },
            {
                'displayKey' => 'Geldbetrag',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12.345,6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Money'
            },
            {
                'displayKey' => 'Text',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'Blue Angel',
                        'value' => 'Blue Angel'
                    }
                ],
                'type' => 'string',
                'key' => 'String'
            },
            {
                'displayKey' => 'Datei',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => $ImageBasePath.'/image.png',
                        'value' => $ImageBasePath.'/image.png'
                    }
                ],
                'type' => 'url',
                'key' => 'File'
            },
            {
                'displayKey' => "Datei, sprachabh\x{e4}ngig",
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => $ImageBasePath.'/Handbuch.pdf',
                        'value' => $ImageBasePath.'/Handbuch.pdf'
                    }
                ],
                'type' => 'url',
                'key' => 'LocalizedFile'
            },
            {
                'displayKey' => 'Ja/Nein',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'true',
                        'value' => 'true'
                    }
                ],
                'type' => 'boolean',
                'key' => 'Boolean'
            },
            {
                'displayKey' => 'Datum',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22.07.2014',
                        'value' => '2014-07-22'
                    }
                ],
                'type' => 'date',
                'key' => 'Date'
            },
            {
                'displayKey' => 'Zeit',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '09:58',
                        'value' => '09:58:00'
                    }
                ],
                'type' => 'time',
                'key' => 'Time'
            },
            {
                'displayKey' => 'Datum und Zeit',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22.07.2014 09:58',
                        'value' => '2014-07-22T09:58:05+02:00'
                    }
                ],
                'type' => 'datetime',
                'key' => 'DateTime'
            },
            {
                'displayKey' => "Mehrfachauswahl, nicht sprachabh\x{e4}ngig",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Value 1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Value 2',
                        'value' => 'Value 2'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiString'
            },
            {
                'displayKey' => "Mehrfachauswahl, sprachabh\x{e4}ngig",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Wert 1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Wert 3',
                        'value' => 'Value 3'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiLocalizedString'
            }
        ]
    }, 'GET MyJacket de_DE' ) or Test::More::diag Dumper($hData);
}

sub test_get_master_en {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::CustomAttributes->new;
    my $ProductFolder = $self->shop->get('ProductFolder');

    my $Product = $ProductFolder->child('MyJacket');
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $ImageBasePath = $SSLDomain.$Product->get('WebPath');

    my $hData = $Resource->get( $Product, 'en_AU' );

    is_deeply( $hData, {
        'items' => [
            {
                'displayKey' => 'LocalizedString',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '',
                        'value' => undef
                    }
                ],
                'type' => 'string',
                'key' => 'LocalizedString'
            },
            {
                'displayKey' => 'Integer',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '123,456,789',
                        'value' => '123456789'
                    }
                ],
                'type' => 'number',
                'key' => 'Integer'
            },
            {
                'displayKey' => 'Float',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12,345.6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Float'
            },
            {
                'displayKey' => 'Money',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12,345.6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Money'
            },
            {
                'displayKey' => 'String',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'Blue Angel',
                        'value' => 'Blue Angel'
                    }
                ],
                'type' => 'string',
                'key' => 'String'
            },
            {
                'displayKey' => 'File',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => $ImageBasePath.'/image.png',
                        'value' => $ImageBasePath.'/image.png'
                    }
                ],
                'type' => 'url',
                'key' => 'File'
            },
            {
                'displayKey' => "LocalizedFile",
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '',
                        'value' => undef,
                    }
                ],
                'type' => 'url',
                'key' => 'LocalizedFile'
            },
            {
                'displayKey' => 'Boolean',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'true',
                        'value' => 'true'
                    }
                ],
                'type' => 'boolean',
                'key' => 'Boolean'
            },
            {
                'displayKey' => 'Date',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22/07/2014',
                        'value' => '2014-07-22'
                    }
                ],
                'type' => 'date',
                'key' => 'Date'
            },
            {
                'displayKey' => 'Time',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '9:58 AM',
                        'value' => '09:58:00'
                    }
                ],
                'type' => 'time',
                'key' => 'Time'
            },
            {
                'displayKey' => 'DateTime',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22/07/2014 9:58 AM',
                        'value' => '2014-07-22T09:58:05+02:00'
                    }
                ],
                'type' => 'datetime',
                'key' => 'DateTime'
            },
            {
                'displayKey' => "PreDefMultiString",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Value 1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Value 2',
                        'value' => 'Value 2'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiString'
            },
            {
                'displayKey' => "PreDefMultiLocalizedString",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Value1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Value 3',
                        'value' => 'Value 3'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiLocalizedString'
            }
        ]
    }, 'GET MyJacket en_AU' ) or Test::More::diag Dumper($hData);
}


sub test_get_sub_de {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::CustomAttributes->new;
    my $ProductFolder = $self->shop->get('ProductFolder');

    my $Product = $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketRedL');
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $ImageBasePath = $SSLDomain.$Product->get('WebPath');

    my $hData = $Resource->get( $Product, 'de_DE' );

    is_deeply( $hData, {
        'items' => [
            {
                'displayKey' => 'Text, sprachabhngig',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'Achtung: Restposten. Jetzt zugreifen!',
                        'value' => 'Achtung: Restposten. Jetzt zugreifen!'
                    }
                ],
                'type' => 'string',
                'key' => 'LocalizedString'
            },
            {
                'displayKey' => 'Ganze Zahl',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '123.456.789',
                        'value' => '123456789'
                    }
                ],
                'type' => 'number',
                'key' => 'Integer'
            },
            {
                'displayKey' => 'Dezimalzahl',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12.345,6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Float'
            },
            {
                'displayKey' => 'Geldbetrag',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '12.345,6789',
                        'value' => '12345.6789'
                    }
                ],
                'type' => 'number',
                'key' => 'Money'
            },
            {
                'displayKey' => 'Text',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'Blue Angel',
                        'value' => 'Blue Angel'
                    }
                ],
                'type' => 'string',
                'key' => 'String'
            },
            {
                'displayKey' => 'Datei',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => $ImageBasePath.'/image.png',
                        'value' => $ImageBasePath.'/image.png'
                    }
                ],
                'type' => 'url',
                'key' => 'File'
            },
            {
                'displayKey' => "Datei, sprachabh\x{e4}ngig",
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => $ImageBasePath.'/Handbuch.pdf',
                        'value' => $ImageBasePath.'/Handbuch.pdf'
                    }
                ],
                'type' => 'url',
                'key' => 'LocalizedFile'
            },
            {
                'displayKey' => 'Ja/Nein',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => 'false',
                        'value' => 'false'
                    }
                ],
                'type' => 'boolean',
                'key' => 'Boolean'
            },
            {
                'displayKey' => 'Datum',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22.07.2014',
                        'value' => '2014-07-22'
                    }
                ],
                'type' => 'date',
                'key' => 'Date'
            },
            {
                'displayKey' => 'Zeit',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '09:58',
                        'value' => '09:58:00'
                    }
                ],
                'type' => 'time',
                'key' => 'Time'
            },
            {
                'displayKey' => 'Datum und Zeit',
                'singleValue' => JSON::true,
                'values' => [
                    {
                        'displayValue' => '22.07.2014 09:58',
                        'value' => '2014-07-22T09:58:05+02:00'
                    }
                ],
                'type' => 'datetime',
                'key' => 'DateTime'
            },
            {
                'displayKey' => "Mehrfachauswahl, nicht sprachabh\x{e4}ngig",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Value 1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Value 2',
                        'value' => 'Value 2'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiString'
            },
            {
                'displayKey' => "Mehrfachauswahl, sprachabh\x{e4}ngig",
                'singleValue' => JSON::false,
                'values' => [
                    {
                        'displayValue' => 'Wert 1',
                        'value' => 'Value1'
                    },
                    {
                        'displayValue' => 'Wert 3',
                        'value' => 'Value 3'
                    }
                ],
                'type' => 'string',
                'key' => 'PreDefMultiLocalizedString'
            }
        ]
    }, 'GET JacketRedL d' ) or Test::More::diag Dumper($hData);
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing CustomAttributesResource.xml';
    ImportObjects('CustomAttributesResource.xml', $self->shop->pathString);
}

__PACKAGE__->runScript;
