﻿package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 8;
use Test::Exception;

use Data::Dumper;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::ProductCollection;
use utf8;

## no critic qw( ProhibitMagicNumbers )

sub test {
    my $self = shift;

    $self->test_GetByID;
}

sub test_GetByID {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::ProductCollection->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my @GUIDs = (
        $ProductFolder->child('MyPetrol')->guid,
        'non-existing-guid',
        $ProductFolder->child('MyJacket')->guid,
        $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack5')->guid, # not visible!
        $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2')->guid,
    );

    my $hData = $Resource->get( $self->shop, $LocaleID, $CurrencyID, \@GUIDs );
    my $ahProducts = delete $hData->{'items'};
    is_deeply( $hData, {
        results => 4,
        page => 1,
        resultsPerPage => 100,
    }, 'result.items') or Test::More::diag Dumper($hData, $ahProducts);

    is scalar @$ahProducts, 4, 'number of items';
    is $ahProducts->[0]->{'productId'}, $ProductFolder->child('MyPetrol')->guid, 'items[0]';
    is $ahProducts->[1]->{'productId'}, $ProductFolder->child('MyJacket')->guid, 'items[1]';
    is $ahProducts->[2]->{'productId'}, $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack5')->guid, 'items[2]'; #not visible
    is $ahProducts->[3]->{'productId'}, $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2')->guid, 'items[3]';

    my $hProduct = $ahProducts->[0];
    is_deeply $hProduct->{'priceInfo'}->{'price'}, {
        currency => $CurrencyID,
        taxType => 'GROSS',
        amount => '1.19',
        formatted => '1.19 €',
    }, 'product.priceInfo.price';

    throws_ok sub { $Resource->get( $self->shop, $LocaleID, $CurrencyID, [1..101] ) }, qr/BadRequest/, 'more than 100 ids';
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
