#========================================================================================
# package      DE_EPAGES::ProductPortal::t::PortalSitesByCountry
#----------------------------------------------------------------------------------------
# description  Test TLEs processing related to ProductPortals
#========================================================================================
package DE_EPAGES::ProductPortal::t::PortalSitesByCountry;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 5;

use DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry;
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

sub test {
    my $self = shift;

    $self->testGetPortalSitesByCountry();

}

sub testGetPortalSitesByCountry {
    my $self = shift;

    note 'Test GET_PORTAL_SITES_BY_COUNTRY';

    # Build a wrapper function to use the TLE function "GET_PORTAL_SITES_BY_COUNTRY"
    my $Processor = DE_EPAGES::TLE::API::BaseProcessor->new();
    my $PortalSites = DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry->new();
    $PortalSites->register($Processor);

    #Set languageID for localized attribute NameOrAlias
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $self->shop->get('LanguageID');


    my $fPortalSitesByCountry = sub {
        my ($ShopID) = @_;
        $Processor->callBlock('GET_PORTAL_SITES_BY_COUNTRY', sub {[ $ShopID ]});
    };


    my $PortalSitesByCountry = $fPortalSitesByCountry->($self->shop->id);

    ok(ref($PortalSitesByCountry) eq "ARRAY", 'Expecting to return PortalSitesByCountry ref array for the ShopID');

    cmp_ok(scalar @{$PortalSitesByCountry}, '>=',1, 'Expecting Number of PortalSites to be greater than zero');

    my $PortalSite = shift (@{$PortalSitesByCountry});
    ok(defined( $PortalSite),'Expecting PortalSite to be defined');
    ok(exists( $PortalSite->{'ProductPortalName'}),'Expecting ProductPortalName key to exist');
    ok(exists( $PortalSite->{'PortalSiteIsMultiLanguage'}),'Expecting PortalSiteIsMultiLanguage key to exist');

}

__PACKAGE__->runScript;
