use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

#========================================================================================
# function         ChangeUnityApiUrlScheme
# state            public
#----------------------------------------------------------------------------------------
# syntax           ChangeUnityApiUrlScheme();
#----------------------------------------------------------------------------------------
# description      Changes the use of system attribute "UnityApiUrlScheme"
#                   to the newer pattern.
#========================================================================================
sub ChangeUnityApiUrlScheme {

    my $System = LoadRootObject();
    print "Changing UnityApiUrlScheme for store: ". $System->get('StoreName');

    if ($System->get('UnityUseSubdomainRouting')) {
        my $ProviderDomain = $System->get('UnityUrlScheme');
        $System->set({
            'UnityApiUrlScheme' => "api." . $ProviderDomain
        });
    }
    LogError(GetError()) if ExistsError();
    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { ChangeUnityApiUrlScheme(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Changes the use of system attribute "UnityApiUrlScheme"
    to the newer pattern.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
