package DE_EPAGES::Product::t::LocalesResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 3;

use Data::Dumper;
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::Shop::API::Resource::Locales;

## no critic qw(ProhibitMagicNumbers)

sub init {
    my $self = shift;

    $self->SUPER::init;

    # set up test data
    my $Shop = $self->shop;
    $Shop->featureMaxValue('Languages', 2);

    my $DE = GetPKeyLanguageByCode('de');
    $Shop->addLanguage($DE) unless $Shop->existsLanguage($DE);

    $Shop->removeLocale('en_US');
    $Shop->removeLocale('de_AT');
    $Shop->removeLocale('de_CH');

    $Shop->addLocale('de_AT');
    $Shop->addLocale('en_US');
    $Shop->addLocale('de_CH');
}

sub test {
    my $self = shift;

    my $Shop = $self->shop;

    is scalar @{$Shop->get('SiteLocales')}, 4, 'number of locales'
        or Test::More::diag Dumper([ map { $_->get('LocaleID') } @{$Shop->get('SiteLocales')} ]);

    is $Shop->get('LocaleID'), 'en_GB', 'default locale';

    my $Resource = DE_EPAGES::Shop::API::Resource::Locales->new;
    my $hResult = $Resource->get($Shop);
    is_deeply $hResult, {
        default => 'en_GB',
        items => [qw(en_GB de_AT en_US de_CH)]
    }, 'GET locales' or Test::More::diag Dumper($hResult);
}

__PACKAGE__->runScript;
