use Test::More tests => 1;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw(CorrectPathSlashes ExistsDirectory GetFileNames fsplit);
use Image::Magick;

# Checks if all images have the corrcet file extensions.
RunScript(Sub => sub {

    my $hExtensionToMimeTypes = {
        '.gif' => 'image/gif',
        '.png' => 'image/png',
        '.jpg' => 'image/jpeg',
    };
    my $hMimeTypesToExtension = {
        'image/gif' => '.gif',
        'image/png' => '.png',
        'image/jpeg' => '.jpg',
    };

    my $TestStatus = 1;
    my $DiagMsg;

    my $StylePath = CorrectPathSlashes($ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Styles/Data/Public/SF/ImageSet/');
    if(ExistsDirectory($StylePath)){
        my $aFilePaths = GetFileNames($StylePath, {'recursive' => 1});
        foreach my $FilePath (@$aFilePaths){
            my (undef, undef, $Extension) = fsplit($FilePath);
            if($Extension ne '' && grep { /$Extension/ } qw(.gif .jpg .png)){
                my $Image = Image::Magick->new;
                $Image->Read($FilePath);
                my $ImageFormat = $Image->GetAttribute('mime');
                # Fix for Windows, because the mimetypes contain a x- e.g. 'images/x-gif'.
                $ImageFormat =~ s/x-//ix;
                if($hExtensionToMimeTypes->{$Extension} ne $ImageFormat){
                    $TestStatus = 0;
                    $DiagMsg .= "$FilePath has extension '$Extension' but should be '$hMimeTypesToExtension->{$ImageFormat}'.\n";
                }
            }
        }
    } else {
        $TestStatus = 0;
        $DiagMsg .= "Directory '$StylePath' does not exist.\n";
    }

    ok($TestStatus, 'All images have the correct extension.') or Test::More::diag $DiagMsg;

}, Debug=>1);
