#==============================================================================
# package      DE_EPAGES::TaxUS::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::TaxUS::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Tax::API::Table::TaxRate qw (GetInfoTaxRate ExistsTaxRate);
use DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS qw(GetTaxRateOrRangeByZipCodeCityState);    ## no critic qw(RequireCorrectImports)
use constant PERCENT      => 100.00;
use constant US_COUNTRYID => 840;

#ERRORDEFINITION
sub ErrorDefinition {
    return {};
}

#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================
sub ViewTaxMatrixUS {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # combined und none zuerst anzeigen, dann den Rest
    my $TaxMatrixUS = $Shop->get('TaxMatrix');
    return unless defined $TaxMatrixUS;

    # collect names, ordered by Position and Alias
    my $aTaxClasses = $TaxMatrixUS->get('TaxClasses');
    my $aTaxAreas   = $TaxMatrixUS->get('TaxAreas');

    my @TaxAreas;
    my $Counter = 0;
    foreach my $TaxAreaUS (@$aTaxAreas) {

        my $Info = $TaxAreaUS->tleHash;
        my @TaxClasses;

        # do not display settings for rates for the rest of the world (Rest of world)
        my $IsRestOfWorld = $TaxAreaUS->get('Alias') eq 'Rest of world' ? 1 : 0;
        $Info->{'TaxAreaReadOnly'} = $IsRestOfWorld;

        # test if we have an areaaddress andget default SalesTaxRate
        my $PresenceAddress = $TaxAreaUS->get('PresenceAddress');
        my $hInfoRange      = {};
        if (defined $PresenceAddress and $PresenceAddress->get('CountryID') eq US_COUNTRYID) {

            # get range, if destination based otherwise get value
            my $hParams;
            $hParams->{'ZipCode'} = $PresenceAddress->get('Zipcode');
            $hParams->{'City'}    = $PresenceAddress->get('City');
            $hParams->{'State'}   = $PresenceAddress->get('State');
            $hInfoRange           = GetTaxRateOrRangeByZipCodeCityState($hParams);
        }

        foreach my $TaxClassUS (@$aTaxClasses) {
            my $TaxClassHash = $TaxClassUS->tleHash;

            $TaxClassHash->{'TaxRateUS'} = 0 if $IsRestOfWorld;

            if ((not $IsRestOfWorld) and ExistsTaxRate($TaxAreaUS->id, $TaxClassUS->id)) {
                $TaxClassHash->{'TaxRateUS'} = PERCENT * GetInfoTaxRate($TaxAreaUS->id, $TaxClassUS->id)->{'Rate'};
            }

            # use info from zip2tax table (range or value) for defaults
            $TaxClassHash->{'DefaultTaxRateUSMin'} = $hInfoRange->{'SalesTaxRateMin'};
            $TaxClassHash->{'DefaultTaxRateUSMax'} = $hInfoRange->{'SalesTaxRateMax'};

            $TaxClassHash->{'ErrorItemNo'} = $Counter++;
            push @TaxClasses, $TaxClassHash;
        }
        $Info->{'TaxClasses'} = \@TaxClasses;
        push @TaxAreas, $Info;
    }
    $Servlet->params('TaxAreas', \@TaxAreas);
    $Servlet->params('TaxClasses', [map { $_->tleHash } @$aTaxClasses]);

    return;
}

sub ViewTaxTypes {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $TaxMethod = $Shop->get('TaxMethod');

    # choose the correct ViewAction if we have TaxUS because we use an own template set
    $Servlet->vars('ViewAction', 'MBO-ViewTaxTypes') unless ((defined $TaxMethod) and $TaxMethod->instanceOf('TaxMethodTaxUS'));

    return $self->SUPER::View($Servlet);

}

sub UnityViewTaxUSCalculation {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

1;
