use strict;
use Test::More tests => 10;

my $TestTool = '/var/epages/WebInterface/TestServerConfig';
ok( -x $TestTool, "$TestTool is executable" );

sub TestToolOutput {
    my ($Filename, $ResultExpr, $InverseCheck) = @_;

    my $CommandLine = "$TestTool -f $Filename";
    my $Result = `$CommandLine`;

    return 1 if(($InverseCheck && $Result !~ $ResultExpr) || $Result =~ $ResultExpr);

    Test::More::diag $Result;
    return 0;
}

## no critic (ProhibitComplexRegexes)
ok(TestToolOutput('TestServerConfig1.xml', qr{^no errors found$}), 'Extended ServerConfig.xml');
ok(TestToolOutput('TestServerConfig2.xml', qr{^no errors found$}), 'Minimal ServerConfig.xml');
ok(TestToolOutput('TestServerConfig3.xml',
    qr{^\[CONFIG ERROR\] loading config file\: CNodeValidator::validateChildNodes\(\)\: validation failed.}),
    'Empty ServerConfig.xml');

ok(TestToolOutput('TestServerConfig4.xml', qr{Value \'inv\@lid\' in \'host\' is not a valid IP address or hostname$}),
    'Invalid host name');

ok(TestToolOutput('TestServerConfig5.xml', qr{no errors found$}, 1), 'Broken XML file');
ok(TestToolOutput('TestServerConfig6.xml', qr{\[CONFIG ERROR\].+checkUniquePorts}), 'Ports not unique');
ok(TestToolOutput('TestServerConfig7.xml', qr{^WARNING: no AppServers in pool DefaultPool}), 'no AppServers');
ok(TestToolOutput('TestServerConfig8.xml', qr{\[CONFIG ERROR\].+checkPools}), 'no RequestRouters');
ok(TestToolOutput('TestServerConfig9.xml', qr{^no errors found$}), 'IPv6 addresses');
